****************************************************************************************************
*                                           PROGRAM OVERVIEW
****************************************************************************************************
*
* PROGRAM: ms_irr.sas  
*
* Created (mm/dd/yyyy): 11/05/2014
* Last modified: 11/05/2014
* Version: 1.0
*
*--------------------------------------------------------------------------------------------------
* PURPOSE:
*  The incidence rate ratio (IRR) tool introduces an automated comparison of two cohorts and their 
*  incidence rates. Cohorts are identified by the user specified Cohort Identification and Descriptive 
*  Analysis (CIDA) Tool input and the tool utilizes the output from the CIDA output dataset. The tool 
*  is useful in quickly assessing both the crude and adjusted incidence rate ratios for the two groups 
*  by producing the incidence rate ratio estimates and their corresponding 95% confidence intervals.                                     
*   
*  Program inputs:                                                                                   
*  The "t2_cida.sas7bdat" SAS dataset that is produced using the CIDA tool.
*                                      
*  Program outputs:                                                                                  
*  SAS data file (.sas7bdat format) with the following for each group of interest:  
*     -Number of new users                                                        
*     -Person years of follow up                                                      
*     -Number of events                                                           
*     -Incidence per 1,000 persons                                                
*     -Incidence rate per 1,000 person years                                      
*     -Incidence rate ratio (95% Confidence Interval)                             
*
*  CSV data file (.csv format) with the following for each group of interest:   
*     -Number of new users                                                        
*     -Person years of follow up                                                      
*     -Number of events                                                           
*     -Incidence per 1,000 persons                                                
*     -Incidence rate per 1,000 person years                                      
*     -Incidence rate ratio (95% Confidence Interval)                             
*                                                      
*                                                                                                  
*  PARAMETERS:                                                                       
*      DATAPATH:    Path that includes the output to the MSOC tables from CIDA                          
*      OUTPATH:     Path desired for output of SAS and CSV dataset                                     
*      DPLIST:      List of datapartners of interest                                                   
*      RUNID:       MPR/RUNID version identifier                                                       
*      TREATMENT:   Treatment group identifier                                                         
*      CONTROL:     Control group identifer                                                            
*      EVENT_VAR:	  Inicates which variable to use to define number of events (EpswEvents, AllEvents)
*      ADJUSTVARS:  List of primary variables user wants to adjust for within IRR calculation.         
*                   These include variables that can not be will always be populated and contain no    
*                   missing cells.  User can control for any combination of the following variables:   
*                     -AgeGroup (Age at event)                                                            
*                     -Sex (Gender)                                                                       
*                     -DPID (Data partner site)                                                           
*                     -Year (Year event occured)                                                          
*                     -CCIELIXGRP (pre-index cci score group - using all 17 comorbidities)                
*                     -NumVisits (Medical Visit ranges during pre-specified time period)                  
*                     A note will be output in the log indicating which variables were used in the       
*                     adjustement.																		                 
*      OUTDATA:     Name of SAS and .CSV output                                                       
*
*
*--------------------------------------------------------------------------------------------------
* CONTACT INFO: 
*  Sentinel Coordinating Center
*  info@mini-sentinel.org
*
*--------------------------------------------------------------------------------------------------
*  CHANGE LOG: 
*
*   Version   Date       Initials      Comment (reference external documentation when available)
*   -------   --------   --------   ---------------------------------------------------------------
*             mm/dd/yy
*
***************************************************************************************************;


%macro MS_IRR( DATAPATH=, 
               OUTPATH=, 
               DPLIST=, 
               RUNID=, 
               TREATMENT=, 
               CONTROL=, 
               EVENT_VAR=, 
               ADJUSTVARS=, 
               OUTDATA=);

%put =====> MACRO CALLED: ms_irr v1.0;

***1) Create macrovariable that contains the number of datapartners in datapartner list;
options mlogic mprint;
%let DpCnt=%sysfunc(countw(&DpList));
%put &DpCnt;

/*make treatment and control macro variables lowcase*/
/*This is to help eliminate case sensitivity for group variables - user can input capital or undercase letters*/
/*please refer to line 132*/
%let treatment_low=%lowcase(&treatment.);
%let control_low=%lowcase(&control.);

/*load outpath library for output datasets, library will be called outdata for user identification*/
libname outdata "&outpath.";




***2)load each individual data partner library;
%macro load_lib (site);
%do k=1 %to &DpCnt.;

	libname %scan(&dplist.,&k.) "&datapath.\%scan(&dplist.,&k.)\MSOC";

%end;
%mend load_lib;
%load_lib();




***3) Extract relevant data from CIDA output and clean data;
%macro clean ();
%do k=1 %to &dpcnt.;
	data %scan(&dplist.,&k.)_clean;
		set %scan(&dplist.,&k.).&RUNID._t2_cida ;
		
		/*create data partner variable*/
		dpid=&k.;

		/*	Have to hard code these strata because all of these need to be populated	*/
		if level in ('002' ,'005', '105', '205', '305') ;
		
		/* Make sure there are patients in the level */
		if (Npts > 0) ;

		/*remove claims with unknown sex values	*/
		if (sex in ("M", "F"));

        if level in ('002') then SEX="T";  *for creating accurate overall total after removing non M and F;

    keep dpid group level sex agegroup year numvisits ccielixgrp Npts tte &event_var. ;
	run;


    proc summary missing nway data = %scan(&dplist.,&k.)_clean ;
    class group level &adjustvars. ;  
    var tte Npts &event_var. ;
    output out = %scan(&dplist.,&k.)_clean_summ (drop = _:) SUM = ;
    run ;

    /*restricting Total Npts to level 002 (M+F)*/
    data %scan(&dplist.,&k.)_clean_summ(rename=Num_pts=Npts);
    set %scan(&dplist.,&k.)_clean_summ; 
    if level in ('002') then Num_pts=Npts;
    retain Num_pts;
    drop Npts;
    run;

    /*to avoid summing over lines after*/
    data %scan(&dplist.,&k.)_clean_summ;
    set %scan(&dplist.,&k.)_clean_summ; 
    by group level;
    if not first.level then Npts=.;
    if level = '002' then delete;
    run;

%end ;

%mend clean;
%clean ();




***4) stack all datasets to form one complete dataset representing all data partners;
%macro stack ();
data stack1;
	set 
	%do k=1 %to &dpcnt.;
			%scan(&dplist.,&k.)_clean_summ
	%end;
	;


	/*create years at risk*/
	years_at_risk=(tte/365);
	/*create log exposure time for poisson offset*/
	log_exposure_time=log(years_at_risk);
	/*create new all lowercase group variable indicator*/
	/*will be used with lowecase macro variables created on lines 82 and 83*/
	group_new=lowcase(group);
	/*subset to groups of interest*/
	if group_new="&treatment_low." | group_new="&control_low.";
	/* Create the events varible of interest*/
    NumEvents = &event_var. ;
	/*create binary indicator for groups*/
	indicator=1;
	if group_new="&treatment_low." then indicator=0;
run;

%mend stack;
%stack ();


***5) Keep the most granular level of summarization ;
***   that exists ;

proc sql noprint ;

  select max(level) into :max_level
  from stack1 ;
  
quit ;

data stack ;
  set stack1 ;
  where (level eq "&max_level.") ;
run ;

proc summary missing nway data = stack ;
class indicator ;
var npts years_at_risk numevents ;
output out = test (drop = _:) SUM = ;
run ;

proc print data = test ;
run ;

***6) run poisson regression for unadjusted IRR;

proc genmod data=stack;
   class  indicator  / param=ref;
   model NumEvents = indicator  / offset= log_exposure_time dist=poisson link=log;
   ods output ParameterEstimates=unadjusted_irr (where=(parameter="indicator") 
			keep=parameter estimate LowerWaldCL UpperWaldCL);
run;

/*clean output and take exponential of estimates*/

data unadjusted_irr_clean;
	set unadjusted_irr;
	parameter_ratio="&treatment./&control.";
	crude_irr=exp(estimate);
	crude_lower_CI=exp(LowerWaldCL);
	crude_upper_CI=exp(UpperWaldCL);
	drop parameter estimate LowerWaldCL UpperWaldCL;
run;




***8) run poisson regression for adjusted IRR;
proc genmod data=stack;
   class  indicator &adjustvars. / param=ref;
   model NumEvents = indicator &adjustvars. / offset= log_exposure_time dist=poisson link=log;
   ods output ParameterEstimates=adjusted_irr (where=(parameter="indicator") 
			keep=parameter estimate LowerWaldCL UpperWaldCL);
run;

/*clean output and take exponential of estimates*/

data adjusted_irr_clean;
	set adjusted_irr;
	parameter_ratio="&treatment./&control.";
	adjusted_irr=exp(estimate);
	adjusted_lower_CI=exp(LowerWaldCL);
	adjusted_upper_CI=exp(UpperWaldCL);
	drop parameter estimate LowerWaldCL UpperWaldCL;
run;


***9) obtain the following for each cohort
	-number of new users
	-number of follow up person years
	-number of outcome events
	-incidence per 1,000 persons
	-incidence rate per 1,000 person years;

proc means data=stack nway noprint;
	class group_new;
	var Npts numevents years_at_risk;
	output out=all_stats (drop=_:) sum=;
run;

/*compute incidence per 1,000 persons and incidence rate per 1,000 person years*/
data treatment_stats_IRR control_stats_IRR;
	set all_stats;
	/*incidence per 1,000 persons*/
	incidence_per_1000=numevents/(npts/1000);
	/*IRR per 1,000 person years*/
	IR_per_1000=numevents/(years_at_risk/1000);
	if group_new="&treatment_low." then output treatment_stats_irr;
	else output control_stats_irr;
run;

/*clean data and rename variable names*/
data trt_stats_irr_clean;
	set treatment_stats_IRR;
	parameter_ratio="&treatment./&control.";
	trt_new_users=npts;
	trt_numevents=numevents;
	trt_person_years=years_at_risk;
	trt_incidence_per_1000=incidence_per_1000;
	trt_IR_per_1000=IR_per_1000;
	drop group_new npts numevents years_at_risk incidence_per_1000 ir_per_1000;
run;

data cntrl_stats_irr_clean;
	set control_stats_IRR;
	parameter_ratio="&treatment./&control.";
	cntrl_new_users=npts;
	cntrl_numevents=numevents;
	cntrl_person_years=years_at_risk;
	cntrl_incidence_per_1000=incidence_per_1000;
	cntrl_IR_per_1000=IR_per_1000;
	drop group_new npts numevents years_at_risk incidence_per_1000 ir_per_1000;
run;


***10) Merge all data sets and export
	-unadjusted_irr_clean
	-adjusted_irr_clean
	-trt_stats_irr_clean
	-cntrl_stats_irr_clean;

data outdata.&outdata.;
	retain MPR_RunID;
	merge trt_stats_irr_clean cntrl_stats_irr_clean unadjusted_irr_clean adjusted_irr_clean;
	by parameter_ratio;
	MPR_RunID="&runid.";
run;

proc export data=outdata.&outdata.
	outfile="&outpath.\&outdata..csv"
	dbms=csv
	replace;
run;


*** 11) Add to log a note about which adjustvars were used ;
%put NOTE: The variables - &adjustvars. were used for adjustment. ;


%put NOTE: ********END OF MACRO: ms_irr v1.0********;


%mend MS_IRR;

